%% Please change your folder for LiDAR metrics file
sDir = 'YourDataFolder';
%% LiDAR metrics file in excel format
sFile = 'linkedLidarMetricsLAI.xls';

%% Please change the output folder and file
outDir = 'YourOutputFolder\result';
outFile = 'plsr_result.xls';

[dataValue,dataTxt] = xlsread(fullfile(sDir,sFile));
LAImax = max(dataValue(:,1));
LAImin = min(dataValue(:,1));
y = ( dataValue(:,1));%% response variable, LAI

nMetrics = size(dataValue,2);%% the number of data column
beginMetrics = 4;% predictor variables starts from hP01 which is in Col 4

X = dataValue(:,beginMetrics:nMetrics);%  predictor variables

n = size(y,2);
nX = size(X,2);

%% set a init number of PCs
%% this number will be changed according to the performance of model
initCom = 15;

%% build the model and inspect the MSE with the increase of factor from 1 to 15
%% the MSE is calcualted using 5-fold cross validation ( cv = 5)
[XL,yl,XS,YS,beta,PCTVAR,MSE,stat] = plsregress(X,y,initCom,'cv',5,'mcreps',5);

%% Plot the percent of variance explained in the response variable as a function of the number of components.
figure(1);
sumPCTVAR = cumsum(100*PCTVAR,2);
plot(1:initCom,sumPCTVAR(2,:),'-bo');
xlabel('Number of PLS components');
ylabel('Percent Variance Explained in y');
%% Plot MSE of the cross-validation
figure(2);
[axes,h1,h2] = plotyy(0:initCom,MSE(1,:),0:initCom,MSE(2,:));
set(h1,'Marker','o')
set(h2,'Marker','o')
legend('MSE Predictors','MSE Response')
xlabel('Number of Components')

%% according to the Figure 1 and Figure 2, the user can input the optimum number of PCs
selCom = input('Please input the number of factors(default is 1): ');
if isempty(selCom)
    selCom = 1;
end

%% the final model will be validated using new data by Leave-out-one cross-validation
%% validate using leave-out-one cross validation
nCV = size(X,1);
CVO = cvpartition(nCV,'KFold',nCV);%% leave out one
N = CVO.NumTestSets;
%% init some variables
err = zeros(N,1);
va_y = zeros(N,1);%% to be used as validation Y
yfit_v = zeros(N,1); %% fitted Y

for i = 1:N
    trIdx = CVO.training(i);
    vaIdx = CVO.test(i);
    trX = X(trIdx,:);
    tr_y = y(trIdx);
    vaX = X(vaIdx,:);
    va_y(i) = y(vaIdx);    
    [XL,yl,XS,YS,beta_v] = plsregress(trX,tr_y,selCom);
    yfit_v(i) = [ones(size(vaX,1),1) vaX]*beta_v;
    err(i) = abs(yfit_v(i) - va_y(i));
end
%% then statistics on the validation result
meanErr = mean(err);
TSS = sum((va_y-mean(va_y)).^2);
RSS = sum((va_y-yfit_v).^2);
Rsquared = 1 - RSS/TSS;
figure(3)
scatter(va_y,yfit_v);
rmse = sqrt(RSS/N);
title(['R^2=' num2str(Rsquared,'%4.2f\n') ' RMSE=' num2str(rmse,'%4.2f\n')]);
%% if there is high correlationship between predicted LAI and true LAI, there will be a very small p
[r, p] = corrcoef(va_y,yfit_v);
P = p(1,2);
disp(['p = ' , num2str(P)]);
if P < 0.05 
    disp('the prediction is significant')
end
%% if the above CV is satisfied, then the model can be used to estimate LAI
[XL,yl,XS,YS,beta_v,PCTVAR,MSE,stat] = plsregress(X,y,selCom);
%% calculate the plot scale LAI
yfit = [ones(size(X,1),1) X]*beta;
sumPCTVAR = cumsum(100*PCTVAR,2); 
%% write the final result into excel file
fileName = fullfile(outDir,outFile);
xlswrite(fileName,[PCTVAR;sumPCTVAR]','pctvar');
xlswrite(fileName,[stat.W],'weight');
xlswrite(fileName,MSE','MSE');
xlswrite(fileName,[y,yfit],'fittedLAI')
